import logging
from datetime import timedelta
from typing import Any  # noqa: F401

from airflow.models import DAG
from airflow.providers.google.cloud.operators.bigquery import (
    BigQueryExecuteQueryOperator,
    BigQueryInsertJobOperator,
)
from airflow.utils.dates import days_ago

logging.warning("Inside the DAG")

args = {"owner": "airflow", "start_date": days_ago(10)}

dag = DAG(
    dag_id="bigquery_musicbrainz_etl",
    default_args=args,
    schedule_interval="0 0 1 * *",
    dagrun_timeout=timedelta(minutes=60),
)

# On GCP Compose "bigquery_default" works, so we call it like this
# instead of "test_fernet"
ALVIN_GCP_CONN_ID = "bigquery_default"


def task_failure_func(context):
    print(f"Alvin callback - {task_failure_func.__name__}: {context}")


def task_success_func(context):
    print(f"Alvin callback - {task_success_func.__name__}: {context}")


def task_retry_func(context):
    print(f"Alvin callback - {task_retry_func.__name__}: {context}")


TASK1_BQ_SQL_QUERY = """
CREATE OR REPLACE TABLE `alvinai.musicbrainz.recordings_by_artists_manual`
AS SELECT artist.id, artist.gid as artist_gid,
       artist.name as artist_name, artist.area,
       recording.name as recording_name, recording.length,
       recording.gid as recording_gid, recording.video
  FROM `alvinai.musicbrainz.artist` as artist
      INNER JOIN `alvinai.musicbrainz.artist_credit_name` AS artist_credit_name
           ON artist.id = artist_credit_name.artist
      INNER JOIN `alvinai.musicbrainz.recording` AS recording
           ON artist_credit_name.artist_credit = recording.artist_credit
"""

task1 = BigQueryInsertJobOperator(
    dag=dag,
    task_id="01_refresh_artists",
    configuration={"query": {"query": TASK1_BQ_SQL_QUERY, "useLegacySql": False}},
    gcp_conn_id=ALVIN_GCP_CONN_ID,
    on_failure_callback=task_failure_func,
    on_success_callback=task_success_func,
    on_retry_callback=task_retry_func,
)

TASK2_BQ_SQL_QUERY = """
CREATE OR REPLACE TABLE `alvinai.musicbrainz.healthcheck_recordings_01`
AS SELECT artist_name, artist_gender, artist_area, recording_name, recording_length
FROM musicbrainz.recordings_by_artists_dataflow
WHERE artist_area is NOT NULL
LIMIT 1000;
"""


task2 = BigQueryInsertJobOperator(
    dag=dag,
    task_id="02_healthcheck_recordings_by_artists_dataflow",
    configuration={"query": {"query": TASK2_BQ_SQL_QUERY, "useLegacySql": False}},
    gcp_conn_id=ALVIN_GCP_CONN_ID,
    on_failure_callback=task_failure_func,
    on_success_callback=task_success_func,
    on_retry_callback=task_retry_func,
)

TASK3_BQ_SQL_QUERY = """
CREATE OR REPLACE TABLE `alvinai.musicbrainz.healthcheck_recordings_02`
AS SELECT artist_name, artist_gender, artist_area, recording_name, recording_length
FROM musicbrainz.recordings_by_artists_dataflow
WHERE artist_gender IS NOT NULL
LIMIT 5000;
"""

task3 = BigQueryInsertJobOperator(
    dag=dag,
    task_id="03_healthcheck_recordings_by_artists_dataflow_nested",
    configuration={"query": {"query": TASK3_BQ_SQL_QUERY, "useLegacySql": False}},
    gcp_conn_id=ALVIN_GCP_CONN_ID,
    on_failure_callback=task_failure_func,
    on_success_callback=task_success_func,
    on_retry_callback=task_retry_func,
)

TASK4_BQ_SQL_QUERY = """
CREATE OR REPLACE TABLE `alvinai.musicbrainz.recordings_by_artists_curated` 
AS SELECT artist_name, artist_gender, artist_area, recording_name, recording_length 
FROM musicbrainz.recordings_by_artists_dataflow 
WHERE artist_area is NOT NULL 
AND artist_gender IS NOT NULL 
AND recording_length > 10000 
"""

task4 = BigQueryInsertJobOperator(
    dag=dag,
    task_id="04_refresh_artists_curated",
    configuration={"query": {"query": TASK4_BQ_SQL_QUERY, "useLegacySql": False}},
    gcp_conn_id=ALVIN_GCP_CONN_ID,
    on_failure_callback=task_failure_func,
    on_success_callback=task_success_func,
    on_retry_callback=task_retry_func,
)

TASK5_BQ_SQL_QUERY = """
CREATE OR REPLACE TABLE `musicbrainz.recordings_by_artists_with_area`
AS SELECT artist_name,
       artist_gender,
       artist_area,
       ARRAY(SELECT artist_credit_name_name
               FROM UNNEST(recordings_by_artists_dataflow_nested.artist_recordings)) AS artist_credit_name_name,
       ARRAY(SELECT recording_name
               FROM UNNEST(recordings_by_artists_dataflow_nested.artist_recordings)) AS recording_name
 FROM musicbrainz.recordings_by_artists_dataflow_nested,
      UNNEST(recordings_by_artists_dataflow_nested.artist_recordings) AS artist_recordings_struct
WHERE artist_area IS NOT NULL
LIMIT 1000;
"""

task5 = BigQueryInsertJobOperator(
    dag=dag,
    task_id="05_refresh_a_area",
    configuration={"query": {"query": TASK5_BQ_SQL_QUERY, "useLegacySql": False}},
    gcp_conn_id=ALVIN_GCP_CONN_ID,
    on_failure_callback=task_failure_func,
    on_success_callback=task_success_func,
    on_retry_callback=task_retry_func,
)


TASK6_BQ_SQL_QUERY = """
CREATE OR REPLACE TABLE `musicbrainz.recordings_by_artists_with_area_curated`
AS SELECT * FROM musicbrainz.recordings_by_artists_with_area rbawa
WHERE EXISTS (select 1 from alvinai.musicbrainz.recordings_by_artists_curated rbac where rbawa.artist_name = rbac.artist_name );
"""

task6 = BigQueryInsertJobOperator(
    dag=dag,
    task_id="06_refresh_a_curated",
    configuration={"query": {"query": TASK6_BQ_SQL_QUERY, "useLegacySql": False}},
    gcp_conn_id=ALVIN_GCP_CONN_ID,
    on_failure_callback=task_failure_func,
    on_success_callback=task_success_func,
    on_retry_callback=task_retry_func,
)

TASK7_BQ_SQL_QUERY = """
SELECT * FROM `musicbrainz.recordings_by_artists_with_area_curated` limit 10
"""

task7 = BigQueryExecuteQueryOperator(
    dag=dag, task_id="07_get_artists", use_legacy_sql=False, sql=TASK7_BQ_SQL_QUERY
)

task1 >> task2
task1 >> task3
task2 >> task4
task2 >> task5
task3 >> task5
task5 >> task6
task4 >> task6
task4 >> task7



import time
from datetime import timedelta
from typing import Any, List, NoReturn, Optional, Union  # noqa: F401

from airflow.contrib.operators.bigquery_operator import BigQueryOperator
from airflow.models import DAG
from airflow.providers.google.cloud.operators.bigquery import (
    BigQueryExecuteQueryOperator,
    BigQueryInsertJobOperator,
)
from airflow.utils.dates import days_ago
from airflow.utils.log.logging_mixin import LoggingMixin  # type: ignore

log = LoggingMixin().log

args = {"owner": "airflow", "start_date": days_ago(10)}

dag = DAG(
    dag_id="bigq_demo",
    default_args=args,
    schedule_interval="0 0 1 * *",
    dagrun_timeout=timedelta(minutes=60),
)

task1 = BigQueryInsertJobOperator(
    dag=dag,
    task_id="task1",
    configuration={
        "query": {
            "query": "INSERT bigquery_demo.top_rising_term_max_min_airflow (region, term_name, score) SELECT region, term_name, cast(term_index as STRING), FROM bigquery_demo.top_rising_term_highest UNION ALL SELECT region, term_name, cast(term_index as STRING), FROM `alvinai.bigquery_demo.top_rising_term_lowest` limit 1",
            "useLegacySql": False,
        }
    },
)

task2 = BigQueryExecuteQueryOperator(
    dag=dag,
    task_id="task2",
    use_legacy_sql=False,
    sql="INSERT bigquery_demo.top_rising_term_region_airflow (region, term, term_index,term_gain) SELECT dma_name, term, cast(score as String), cast(percent_gain as String) FROM bigquery_demo.top_rising_term limit 1",
)

task3 = BigQueryOperator(
    dag=dag,
    task_id="task3",
    use_legacy_sql=False,
    sql="INSERT bigquery_demo.top_rising_term_max_min_airflow (region, term_name, score) SELECT region, term_name, cast(term_index as STRING), FROM bigquery_demo.top_rising_term_highest UNION ALL SELECT region, term_name, cast(term_index as STRING), FROM `alvinai.bigquery_demo.top_rising_term_lowest` limit 1",
)
task1 >> task2 >> task3



import airflow


def update_state(self, session=None):
    """
    Determines the overall state of the DagRun based on the state
    of its TaskInstances.
    :return: ready_tis: the tis that can be scheduled in the current loop
    :rtype ready_tis: list[airflow.models.TaskInstance]
    """
    import logging

    from airflow.settings import Stats
    from airflow.ti_deps.dep_context import SCHEDULEABLE_STATES
    from airflow.utils import timezone
    from airflow.utils.state import State

    from alvin_integration.helper import AlvinLoggerAdapter
    from alvin_integration.producers.airflow.lineage.backend import (
        alvin_dag_run_extractor,
    )

    log = AlvinLoggerAdapter(logging.getLogger(__name__), {})

    @airflow.utils.db.provide_session
    def get_session(session):
        return session

    if not session:
        session = get_session()

    dag = self.get_dag()
    ready_tis = []
    tis = [
        ti
        for ti in self.get_task_instances(
            session=session, state=State.task_states + (State.SHUTDOWN,)
        )
    ]
    log.debug("number of tis tasks for %s: %s task(s)", self, len(tis))
    for ti in list(tis):
        ti.task = dag.get_task(ti.task_id)

    start_dttm = timezone.utcnow()
    unfinished_tasks = [t for t in tis if t.state in State.unfinished()]
    finished_tasks = [
        t for t in tis if t.state in State.finished() + [State.UPSTREAM_FAILED]
    ]
    none_depends_on_past = all(not t.task.depends_on_past for t in unfinished_tasks)
    none_task_concurrency = all(
        t.task.task_concurrency is None for t in unfinished_tasks
    )
    # small speed up
    if unfinished_tasks:
        scheduleable_tasks = [
            ut for ut in unfinished_tasks if ut.state in SCHEDULEABLE_STATES
        ]
        log.debug(
            "number of scheduleable tasks for %s: %s task(s)",
            self,
            len(scheduleable_tasks),
        )
        ready_tis, changed_tis = self._get_ready_tis(
            scheduleable_tasks, finished_tasks, session
        )
        log.debug("ready tis length for %s: %s task(s)", self, len(ready_tis))
        if none_depends_on_past and none_task_concurrency:
            # small speed up
            are_runnable_tasks = (
                ready_tis
                or self._are_premature_tis(unfinished_tasks, finished_tasks, session)
                or changed_tis
            )

    duration = timezone.utcnow() - start_dttm
    Stats.timing("dagrun.dependency-check.{}".format(self.dag_id), duration)

    leaf_task_ids = {t.task_id for t in dag.leaves}
    leaf_tis = [ti for ti in tis if ti.task_id in leaf_task_ids]

    # if all roots finished and at least one failed, the run failed
    if not unfinished_tasks and any(
        leaf_ti.state in {State.FAILED, State.UPSTREAM_FAILED} for leaf_ti in leaf_tis
    ):
        log.info("Marking run %s failed", self)
        self.set_state(State.FAILED)
        dag.handle_callback(self, success=False, reason="task_failure", session=session)

    # if all leafs succeeded and no unfinished tasks, the run succeeded
    elif not unfinished_tasks and all(
        leaf_ti.state in {State.SUCCESS, State.SKIPPED} for leaf_ti in leaf_tis
    ):
        log.info("Marking run %s successful", self)
        self.set_state(State.SUCCESS)
        dag.handle_callback(self, success=True, reason="success", session=session)

    # if *all tasks* are deadlocked, the run failed
    elif (
        unfinished_tasks
        and none_depends_on_past
        and none_task_concurrency
        and not are_runnable_tasks
    ):
        log.info("Deadlock; marking run %s failed", self)
        self.set_state(State.FAILED)
        dag.handle_callback(
            self, success=False, reason="all_tasks_deadlocked", session=session
        )

    # finally, if the roots aren't done, the dag is still running
    else:
        self.set_state(State.RUNNING)

    self._emit_true_scheduling_delay_stats_for_finished_state(finished_tasks)
    self._emit_duration_stats_for_finished_state()
    alvin_dag_run_extractor(dag_run=self)
    # todo: determine we want to use with_for_update to make sure to lock the run
    session.merge(self)
    session.commit()

    return ready_tis



import logging
from typing import List

from openlineage.airflow.extractors.base import TaskMetadata
from openlineage.airflow.extractors.bigquery_extractor import BigQueryExtractor
from openlineage.airflow.utils import get_job_name
from openlineage.client.facet import SqlJobFacet
from openlineage.common.provider.bigquery import (
    BigQueryDatasetsProvider,
    BigQueryErrorRunFacet,
    BigQueryFacets,
)

from alvin_integration.helper import AlvinLoggerAdapter
from alvin_integration.models import AlvinBigQueryLineageDetails, AlvinFacet
from alvin_integration.producers.airflow.lineage.extractors.mixins import (
    AlvinAirflowExtractorMixin,
)

log = AlvinLoggerAdapter(logging.getLogger(__name__), {})


class AlvinBigQueryBaseExtractor(AlvinAirflowExtractorMixin, BigQueryExtractor):
    """
    This class contains common logic to handle extraction of BigQuery lineage
    for both Airflow 1 and 2.

    https://cloud.google.com/bigquery/docs/information-schema-jobs
    """

    def __init__(self, operator, *args, **kwargs):
        super().__init__(operator, *args, **kwargs)

    @classmethod
    def get_operator_classnames(cls) -> List[str]:
        raise NotImplementedError

    def get_bigquery_job_id(self, task_instance):
        return self._get_xcom_bigquery_job_id(task_instance)

    def build_task_metadata(self, task_instance):
        if not self.operator.hook:
            return TaskMetadata(
                name=get_job_name(task=self.operator),
                inputs=[],
                outputs=[],
                run_facets={},
            )
        bigquery_job_id = self.get_bigquery_job_id(task_instance)
        context = self.parse_sql_context()
        big_query_client = self.operator.hook.get_client(
            project_id=self.operator.hook.project_id
        )

        stats = BigQueryDatasetsProvider(big_query_client).get_facets(bigquery_job_id)

        return TaskMetadata(
            name=get_job_name(task=self.operator),
            inputs=[ds.to_openlineage_dataset() for ds in stats.inputs],
            outputs=[stats.output.to_openlineage_dataset()] if stats.output else [],
            run_facets=stats.run_facets,
            job_facets={"sql": SqlJobFacet(context.sql)},
        )

    def build_facet(self, task_instance):
        bigquery_job_id = self.get_bigquery_job_id(task_instance)

        return AlvinFacet(
            alvin=AlvinBigQueryLineageDetails(
                job_id=bigquery_job_id,
                project_id=self.operator.hook.project_id
                if self.operator.hook
                else None,
                execution=self.get_execution_details(task_instance),
                connection_id=self.get_connection_id(),
            )
        )


class AlvinBigQueryExtractor(AlvinBigQueryBaseExtractor):
    """
    This class contains logic for Airflow 2.

    Note that on this version, the BigQuery operator class is
    part of this package: apache-airflow-providers-google.

    https://github.com/OpenLineage/OpenLineage/tree/main/integration/airflow#custom-extractors

    The following env var needs to be set in the airflow host environment:
    OPENLINEAGE_EXTRACTOR_BigQueryExecuteQueryOperator
        =alvin_airflow.lineage.extractors.bigquery.AlvinBigQueryExtractor
    """

    # We override this method because not all Operators has the
    # the sql attribute, if the user is using a BigQueryInsertJobOperator
    # a bunch of stacktraces would show up in their logs, which we should avoid.
    # Only override this logic for AirflowV2 to reduce the chance to impact something.
    def parse_sql_context(self):
        try:
            if hasattr(self.operator, "sql"):

                # Here we just call the parent method, so we dont have to
                # copy all the code here.
                return super().parse_sql_context()
        except Exception as e:
            log.debug("Cannot parse sql context!")

        # Only import this SqlContext if we really need to!
        # This object is required and used down the code chain,
        # so we must return something from this method to not break
        from openlineage.airflow.extractors.bigquery_extractor import SqlContext

        return SqlContext(sql="", parser_error="Cannot parse sql context!")

    def get_bigquery_job_id(self, task_instance):
        bigquery_job_id = self._get_xcom_bigquery_job_id(task_instance)

        # we have a fallback logic that takes the job_id from the operator if its not filled
        # inside the xcom, doing some tests we saw this happen in the BigQueryInsertJobOperator
        if not bigquery_job_id and self.operator and hasattr(self.operator, "job_id"):
            bigquery_job_id = self.operator.job_id
        return bigquery_job_id

    def get_airflow_run_id(self, task_instance):
        dag_run = task_instance.get_dagrun()
        return dag_run.run_id

    def get_connection_id(self):
        return self.operator.gcp_conn_id if self.operator else None

    @classmethod
    def get_operator_classnames(cls) -> List[str]:
        return ["BigQueryExecuteQueryOperator", "BigQueryInsertJobOperator"]


class AlvinBigQueryLegacyExtractor(AlvinBigQueryBaseExtractor):
    """
    This class contains logic for Airflow 1.

    Note that on this version, the BigQuery operator class part of the
    Airflow contrib google module.

    https://github.com/OpenLineage/OpenLineage/tree/main/integration/airflow#custom-extractors

    The following env var needs to be set in the airflow host environment:
    OPENLINEAGE_EXTRACTOR_BigQueryOperator
        =alvin_airflow.lineage.extractors.bigquery.AlvinBigQueryLegacyExtractor
    """

    def get_airflow_run_id(self, task_instance):
        return task_instance.get_dagrun().run_id

    def get_connection_id(self):
        return self.operator.bigquery_conn_id if self.operator else None

    @classmethod
    def get_operator_classnames(cls) -> List[str]:
        return ["BigQueryOperator"]



import logging
from datetime import timedelta
from typing import Any  # noqa: F401

from airflow.contrib.operators.bigquery_operator import BigQueryOperator
from airflow.models import DAG
from airflow.utils.dates import days_ago

logging.warning("Inside the DAG")

args = {"owner": "airflow", "start_date": days_ago(10)}

dag = DAG(
    dag_id="bq_mvp",
    default_args=args,
    schedule_interval="0 0 1 * *",
    dagrun_timeout=timedelta(minutes=60),
)

# On GCP Compose "bigquery_default" works, so we call it like this
# instead of "test_fernet"
ALVIN_GCP_CONN_ID = "bigquery_default"


BQ_SQL_QUERY = "SELECT * FROM alvin_test_dataset_eu.WineQualityEU"


taskBQ = BigQueryOperator(
    dag=dag,
    task_id="taskBQ",
    sql=BQ_SQL_QUERY,
    bigquery_conn_id=ALVIN_GCP_CONN_ID,
)



import logging


class AlvinLoggerAdapter(logging.LoggerAdapter):
    """
    This allows for more flexible troubleshooting on Cloud platforms e.g. GCP Composer.
    Each log statement coming from this Alvin plugin has a [alvin] prefix.
    https://docs.python.org/3/howto/logging-cookbook.html#using-loggeradapters-to-impart-contextual-information
    """

    def process(self, msg, kwargs):
        return f"[alvin] {msg}", kwargs



import logging
import os
import pathlib
import traceback

from airflow.lineage.backend import LineageBackend
from openlineage.lineage_backend import Backend

from alvin_integration.helper import AlvinLoggerAdapter

log = AlvinLoggerAdapter(logging.getLogger(__name__), {})


class AlvinAirflowBackend(Backend):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


ALVIN_STATE_NOT_STARTED = "NOT_STARTED"
ALVIN_STATE_FINISHED = "FINISHED"


class AlvinStateSingleton:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(AlvinStateSingleton, cls).__new__(
                cls, *args, **kwargs
            )
        return cls._instance

    def __init__(self):
        self._state = ALVIN_STATE_NOT_STARTED

    def set_state(self, new_value):
        self._state = new_value

    def get_state(self):
        return self._state


class AlvinBackendMeta(type):
    def __init__(cls, *args, **kwargs):
        from airflow.configuration import conf

        backend_str = conf.get("lineage", "backend")
        cls.alvin_state = AlvinStateSingleton()
        if backend_str and cls.__name__ in backend_str:
            cls.copy_alvin_plugin()

    @staticmethod
    def read_alvin_plugin():
        """Returns the alvin plugin path."""
        filepath = os.path.join(
            f"{pathlib.Path(__file__).resolve().parent}", "alvin_plugin.py"
        )

        return filepath

    def is_plugin_not_copied(cls, path):
        if cls.alvin_state.get_state() == ALVIN_STATE_NOT_STARTED:
            cls.alvin_state.set_state(ALVIN_STATE_FINISHED)
            return True
        return False

    def copy_alvin_plugin(cls):
        """Copy the Alvin plugin to the airflow plugins folder."""
        filepath = cls.read_alvin_plugin()
        plugin_path = os.path.join(f'{os.getenv("AIRFLOW_HOME")}', "plugins")
        if not os.path.exists(plugin_path):
            log.info(f"Creating plugins folder: {plugin_path}")
            os.makedirs(plugin_path)
        full_path = os.path.join(f"{plugin_path}", "alvin_plugin.py")
        if cls.is_plugin_not_copied(full_path):
            with open(full_path, "w") as plugin_file:
                with open(filepath, "r") as f:
                    content = f.read()
                    plugin_file.write(content)
            log.info(f"Plugin copied successfully to: {full_path}")
        else:
            log.info(f"Plugin already copied: {full_path}")


class AlvinBackendComposerMeta(AlvinBackendMeta):
    def __init__(self, *args, **kwargs):
        super(AlvinBackendComposerMeta, self).__init__(*args, **kwargs)
        self.patch_scheduler_functions()

    def copy_alvin_plugin(cls):
        plugin_path = os.path.join(f'{os.getenv("AIRFLOW_HOME")}', "plugins")
        full_path = os.path.join(f"{plugin_path}", "alvin_plugin.py")
        if cls.is_plugin_not_copied(full_path):
            from google.cloud import storage

            from alvin_integration.producers.airflow.config import (
                GOOGLE_COMPOSER_BUCKET,
            )

            print(
                f"Creating Alvin DAGs on Google Composer path {GOOGLE_COMPOSER_BUCKET}....."
            )

            filepath = cls.read_alvin_plugin()

            with open(filepath, "r") as f:
                content = f.read()

                client = storage.Client()

                bucket = client.get_bucket(GOOGLE_COMPOSER_BUCKET)

                blob = bucket.blob("plugins/alvin_plugin.py")

                blob.upload_from_string(content)

            print(
                f"Plugin copied successfully to: {GOOGLE_COMPOSER_BUCKET}/plugins/alvin_plugin.py"
            )
        else:
            log.info(f"Plugin already copied: {full_path}")

    def patch_scheduler_functions(cls):
        import gorilla
        import pkg_resources
        from airflow.models.dagrun import DagRun

        airflow_package = pkg_resources.get_distribution("apache-airflow")

        if airflow_package.version in ("2.1.4", "2.1.4+composer"):
            from alvin_integration.producers.airflow.patch.functions_214 import (
                update_state,
            )
        elif airflow_package.version in ("2.2.3", "2.2.5", "2.2.3+composer"):
            from alvin_integration.producers.airflow.patch.functions import update_state
        elif airflow_package.version in ("1.10.15", "1.10.15+composer"):
            from alvin_integration.producers.airflow.patch.functions_legacy import (
                update_state,
            )

        settings = gorilla.Settings(allow_hit=True)
        patch = gorilla.Patch(
            DagRun,
            update_state.__name__,
            update_state,
            settings=settings,
        )

        log.info(f"Installing: {patch}")

        gorilla.apply(patch)

        log.info(f"Patch: {patch} installed successfully.")


class AlvinAirflowLineageBackend(LineageBackend, metaclass=AlvinBackendMeta):
    backend: AlvinAirflowBackend = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @classmethod
    def send_lineage(cls, *args, **kwargs):
        """Alvin implementation of the send_lineage backend."""
        log.info("calling send lineage with args and kwargs")
        if not cls.backend:
            log.info("Lineage backend is none, creating it")
            cls.backend = AlvinAirflowBackend()
        return cls.backend.send_lineage(*args, **kwargs)


class AlvinAirflowLineageBackendComposer(
    AlvinAirflowLineageBackend, metaclass=AlvinBackendComposerMeta
):
    pass


def alvin_callback(context, operator, is_airflow_legacy=False):
    """Alvin callback implementation.

    This function use the OpenLineage implementation of
    the Airflow Backend and call send_lineage with the
    given context.
    """
    try:
        if is_airflow_legacy:
            AlvinAirflowLineageBackend.send_lineage(
                context=context, operator=operator
            )  # noqa
        else:
            lineage_backend = AlvinAirflowLineageBackend()
            lineage_backend.send_lineage(context=context, operator=operator)
    except Exception:
        log.error(f"Alvin Callback Error: {traceback.format_exc()}")


def alvin_dag_run_extractor(dag_run):
    import os

    import requests

    from alvin_integration.producers.airflow.config import (
        ALVIN_BACKEND_API_KEY,
        ALVIN_BACKEND_API_URL,
    )

    if dag_run:
        log.info(
            f"Extracting and sending DagRun information for dag_id={dag_run.dag_id} and run_id={dag_run.run_id}"
        )
        try:
            ALVIN_PLATFORM_ID = os.getenv("ALVIN_PLATFORM_ID")
            alvin_backend_metadata_url = f"{ALVIN_BACKEND_API_URL}/api/v1/lineage"

            payload = {
                "alvin_platform_id": ALVIN_PLATFORM_ID,
                "facets": {
                    "dag_run": {
                        "dag_id": dag_run.dag_id,
                        "run_id": dag_run.run_id,
                        "queued_at": (
                            dag_run.queued_at.isoformat() if dag_run.queued_at else None
                        )
                        if hasattr(dag_run, "queued_at")
                        else None,
                        "execution_date": dag_run.execution_date.isoformat(),
                        "start_date": dag_run.start_date.isoformat(),
                        "end_date": dag_run.end_date.isoformat()
                        if dag_run.end_date
                        else None,
                        "state": dag_run.get_state(),
                    }
                },
            }
            requests.post(
                alvin_backend_metadata_url,
                json=payload,
                headers={"X-API-KEY": ALVIN_BACKEND_API_KEY},
            )
        except Exception as e:
            log.error(f"Error sending message to Alvin: {e}")
    else:
        log.info("Dag run not defined.")



import datetime
import logging
from datetime import timedelta

from airflow.models import DAG
from airflow.providers.snowflake.operators.snowflake import SnowflakeOperator

logging.warning("Inside the DAG")

dag = DAG(
    "snowflake_dag_airflow_v2",
    start_date=datetime.datetime.utcnow(),
    default_args={"snowflake_conn_id": "snowflake_db"},
    schedule_interval="0 0 1 * *",
    tags=["example"],
    catchup=False,
    dagrun_timeout=timedelta(minutes=10),
)

task_sf_af_v2 = SnowflakeOperator(
    task_id="task_sf_af_v2",
    dag=dag,
    sql="SELECT 1;",
)

task_sf_af_v2



import pkg_resources

from alvin_integration.producers.airflow.config import AirflowProducerConfig
from alvin_integration.producers.airflow.installer import AlvinAirflowInstaller


def test_setup_airflow_installer():

    alvin_airflow_installer = AlvinAirflowInstaller()

    assert alvin_airflow_installer.host_package_map == {
        "apache-airflow": pkg_resources.get_distribution("apache-airflow")
    }

    assert type(alvin_airflow_installer.provider_config) == AirflowProducerConfig

    assert alvin_airflow_installer.provider_config.producer_name == "Airflow"



from airflow.models import DAG
from airflow.operators.python import PythonOperator


def first_func():
    print("first_func execution")


def second_func():
    print("second_func execution")


dag_task_series = DAG(dag_id="dag_task_series", schedule_interval=None)


first_func_task = PythonOperator(
    task_id="first_func",
    provide_context=True,
    python_callable=first_func,
    dag=dag_task_series,
)

second_func_task = PythonOperator(
    task_id="second_func",
    provide_context=True,
    python_callable=second_func,
    dag=dag_task_series,
)


first_func_task >> second_func_task



import time
from datetime import timedelta
from typing import Any, List, NoReturn, Optional, Union  # noqa: F401

from airflow.models import DAG
from airflow.providers.snowflake.operators.snowflake import SnowflakeOperator
from airflow.utils.dates import days_ago
from airflow.utils.log.logging_mixin import LoggingMixin  # type: ignore

log = LoggingMixin().log
SNOWFLAKE_CONN_ID = "snowflake_db"

args = {
    "owner": "airflow",
    "start_date": days_ago(10),
    "snowflake_conn_id": SNOWFLAKE_CONN_ID,
}

dag = DAG(
    dag_id="snowflake_demo",
    default_args=args,
    schedule_interval="0 0 1 * *",
    dagrun_timeout=timedelta(minutes=60),
)


task1 = SnowflakeOperator(
    task_id="task1",
    dag=dag,
    sql='SELECT SYMBOL FROM "FINSERVAM"."PROD"."FRAUDULENT_TRADE" limit 1',
    warehouse="COMPUTE_WH",
    database="FINSERVAM",
    schema="PROD",
    role="ACCOUNTADMIN",
)

task2 = SnowflakeOperator(
    task_id="task2",
    dag=dag,
    sql='create or replace table fraudulent_trade_airflow as (select date, symbol, exchange, action, close, trader, pm, 0 as fraud from "FINSERVAM"."PROD"."TRADE")',
    warehouse="COMPUTE_WH",
    database="FINSERVAM",
    schema="PROD",
    role="ACCOUNTADMIN",
)


task1 >> task2
